@@ -1,15 +1,15 @@
 package DTL::Fast::Context;
 use strict; use utf8; use warnings FATAL => 'all'; 
-use Carp qw(confess cluck);
+use Carp;
 
 use DTL::Fast::Utils qw(has_method);
 
 sub new
 {
-    my $proto = shift;
-    my $context = shift // {};
+    my( $proto, $context ) = @_;
+    $context //= {};
     
-    confess "Context should be a HASH reference"
+    croak  "Context should be a HASH reference"
         if ref $context ne 'HASH';
         
     return bless {
@@ -19,12 +19,11 @@ sub new
 
 sub get
 {
-    my $self = shift;
-    my $variable_path = shift;
+    my( $self, $variable_path ) = @_;
 
     if( ref $variable_path ne 'ARRAY' )    # suppose that raw variable invoked, period separated
     {
-        $variable_path = [split /\.+/, $variable_path];
+        $variable_path = [split /\.+/x, $variable_path];
     }
     else
     {
@@ -39,8 +38,10 @@ sub get
         $namespace->{$variable_name}
         : undef; 
 
-    $variable = $variable->($self) 
-        if ref $variable eq 'CODE';
+    while( ref $variable eq 'CODE' )
+    {
+        $variable = $variable->($self) 
+    }
         
     $variable = $self->traverse($variable, $variable_path)
         if 
@@ -53,9 +54,7 @@ sub get
 # tracing variable path
 sub traverse
 {
-    my $self = shift;
-    my $variable = shift;
-    my $path = shift;
+    my( $self, $variable, $path ) = @_;
 
     foreach my $step (@$path)
     {
@@ -66,7 +65,7 @@ sub traverse
         }
         elsif( 
             $current_type eq 'ARRAY'
-            and $step =~ /^\-?\d+$/
+            and $step =~ /^\-?\d+$/x
         )
         {
             $variable = $variable->[$step];
@@ -75,39 +74,43 @@ sub traverse
         {
             $variable = $variable->$step($self);
         }
+        elsif( $current_type )
+        {
+            $variable = $variable->{$step};
+        }
         else
         {
-            confess sprintf("Don't know how to traverse %s (%s) with %s"
+            croak  sprintf("Don't know how to traverse %s (%s) with %s"
                 , $variable
                 , $current_type
                 , $step 
             );
         }        
     }
-        
-    $variable = $variable->($self) 
-        if ref $variable eq 'CODE';
+
+    while( ref $variable eq 'CODE' )
+    {
+        $variable = $variable->($self);
+    }
 
     return $variable;
 }
 
 sub set
 {
-    my $self = shift;
-    my @sets = @_;
+    my( $self, @sets ) = @_;
     
-    while( 
-        defined (my $key = shift @sets)
-        and defined (my $val = shift @sets)
-    )
+    while( scalar @sets > 1 )
     {
-        if( $key =~ /\./ )  # traversed set
+        my $key = shift @sets;
+        my $val = shift @sets;
+        if( $key =~ /\./x )  # traversed set
         {
-            my @key = split /\.+/, $key;
+            my @key = split /\.+/x, $key;
             my $variable_name = pop @key;
             my $variable = $self->get([@key]);
             
-            confess sprintf('Unable to set variable %s because parent %s is not defined.'
+            croak  sprintf('Unable to set variable %s because parent %s is not defined.'
                 , $key
                 , join('.', @key)
             ) if not defined $variable;
@@ -119,14 +122,14 @@ sub set
             }
             elsif( 
                 $variable_type eq 'ARRAY'
-                and $variable_name =~ /^\-?\d+$/
+                and $variable_name =~ /^\-?\d+$/x
             )
             {
                 $variable->[$variable_name] = $val;
             }
             else
             {
-                confess sprintf("Don't know how to set variable %s for parent node of type %s"
+                croak  sprintf("Don't know how to set variable %s for parent node of type %s"
                     , $variable_name
                     , $variable_type
                 );
@@ -140,18 +143,18 @@ sub set
     return $self;
 }
 
-sub push
+sub push_scope
 {
-    my $self = shift;
+    my( $self ) = @_;
 #    push @{$self->{'ns'}}, {}; # multi-level version, suppose it's slower on reading
     push @{$self->{'ns'}}, {%{$self->{'ns'}->[-1] // {}}};
     return $self;
 }
 
-sub pop
+sub pop_scope
 {
-    my $self = shift;
-    confess "It's a last context layer available."
+    my( $self ) = @_;
+    croak  "It's a last context layer available."
         if scalar @{$self->{'ns'}} ==  1;
     pop @{$self->{'ns'}};
     return $self;
